%% est_wals.m
%
% version: 2.1
% originally created: 3 april 2008
% this revision: 18 may 2011
%
% Description: Calculates WALS estimates and precisions
% when some regressors (X1) are always in the model, while model selection
% takes place over the remainder (X2) of the regressors.
%
% NOTE: 
% In the may 2011 revision, the matrices X1 and X2 are weighted such that
% the diagonal elements of X1'*X1 and X2'*M1*X2 are all one.
% The reason for the weighting is that this is numerically more stable and
% also that it makes the routine scale-independent. JM (may 2011)
%
% This code may be used freely, but reference should be made to:
%
% Magnus, J.R., O. Powell, and P. Prufer (2009),
%    ``A comparison of two model averaging techniques with an application 
%    to growth empirics,'' Journal of Econometrics, 154, 139-153.
%
% The authors accept no responsibility for any errors or malfunctions
% in the program.
%
% Tested with Matlab 7.4.0 (R2007a).
%
% Comments and suggestions are welcome
% and should be addressed to magnus@uvt.nl.
%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%
%% Inputs:
%
%     data_file = name of Excel file containing in the first
%                 column (n x 1) observations on the dependant variable "y",
%                 followed by (n x k1) observations on the focus regressors
%                 "X1", then followed by (n x k2) observations on the
%                 auxiliary regressors "X2". 
%
%     n         = number of observations.
%     k1        = number of focus regressors.
%     k2        = number of auxiliary regressors.
%
%  ** Note that (X1:X2) together should contain ALL the regressors,
%     including the constant term (if present).
%
%% Outputs:
%
%     b         = vector of coefficient estimates.
%     se        = standard errors associated with b.
%     other     = not used. 
%     ifail     = errors occurred:
%     ifail(1)  = 0    if k1>=0, k2>=0, k>0, n>k,
%                      otherwise gives number of errors (1,2,3,4);
%     ifail(2)  = 0    if dimensions of data input matrix are correctly specified,
%                      otherwise gives number of errors (1,2);
%     ifail(3)  = 0    if there are no missing data,
%                      otherwise = 1;
%     ifail(4)  = 0    if D is positive definite,
%                      otherwise = 1.

%

%% -----------------------------------------------------------
function [b, se, other, ifail] = est_wals(D, n, k1, k2)

ifail(1) = 4;
ifail(2) = 2;
ifail(3) = 1;
ifail(4) = 1;

%% --- Define total number of regressors
k = k1 + k2;

%% --- Check dimensions
if (k1 >= 0)   ifail(1) = ifail(1) - 1; end
if (k2 >= 0)   ifail(1) = ifail(1) - 1; end
if (k > 0)     ifail(1) = ifail(1) - 1; end
if (n > k)     ifail(1) = ifail(1) - 1; end

if (ifail(1) == 0)
    %% --- Load data
    nn         = size(D,1);
    kk         = size(D,2)-1;
    %% --- Check order of data file
    if (nn == n)   ifail(2) = ifail(2) - 1; end
    if (kk == k)   ifail(2) = ifail(2) - 1; end
    %% --- Check missing data in data file
    if (sum(sum(isnan(D))) == 0) ifail(3) = ifail(3) - 1; end
end

%% --- If conditions are satisfied, proceed
if (ifail(1)+ifail(2)+ifail(3) == 0)
    %% --- Define y, X1, and X2 
    y           = D(:,1);
    X1d         = D(:,2:k1+1);
    X2d         = D(:,k1+2:k+1);
    %% --- if no scaling is required set scale=0
    scale = 1;
    %% --- Scale X1 so that the diagonal elements of X1'X1 are all one
    if (k1>0)
        d1 = diag(X1d'*X1d).^(-1/2);
        D1 = diag(d1);
        if (scale==0) D1 = eye(k1); end
        X1 = X1d * D1;
    end
    if (k2 == 0)
        
        [b1, ci, resid] = regress(y, X1);
        s_sqr           = resid' * resid / (n-k);
        s               = s_sqr^0.5;
        V1r             = inv(X1'*X1);
        varb1           = s^2 * V1r;
        b2              = [];
        varb2           = [];
        ifail(4)        = 0;
        
    else %k2>0
        
        %% - Set up Zd
        Zd          = X2d'*X2d;
        if (k1>0)
                V1r         = inv(X1'*X1);
                VV12        = X1'*X2d;
                Zd          = Zd - VV12' * V1r * VV12;
        end
        
        %% --- Define Z (scaled version of Zd)
        d2 = diag(Zd).^(1/2);
        if (scale==0) d2 = ones(k2); end
        for i=1:k2
        for j=1:k2
            weight=d2(i)*d2(j);
            Z(i,j)=Zd(i,j)/weight;
        end
        end
        %% --- Diagonalize Z
        [P, D]          = eig(Z);
    
        %% --- Check that D is positive definite
        order       = max(size(D));
        lambda      = diag(D);
        tol         = eps; %eps = machine precision
        rank        = sum(lambda>tol);
        if (rank == order) 
            ifail(4) = 0;
        end

        %% --- If D is positive definite, proceed
        if (ifail(4) == 0)
            %% --- Set up X2 so that X2'*M1*X2 = I
            for i=1:k2
            for j=1:k2
            weight=d2(i)*lambda(j)^0.5;
            D2(i,j)=P(i,j)/weight;
            end
            end
            X2              = X2d * D2;
            %% --- OLS of unrestricted model
            X               = [X1 X2];
            [b, ci, resid]  = regress(y, X);
            s_sqr           = resid' * resid / (n-k);
            s               = s_sqr^0.5;

            b2_hat          = b(k1+1:k);
            eta_hat         = b2_hat / s;

            %% --- Moments of Laplace function
            c               = log(2);
            signx           = sign(eta_hat);
            x               = abs(eta_hat);
            g0              = normcdf(-x - c, 0, 1);
            g1              = normcdf( x - c, 0, 1);
            g2              = normpdf( x - c, 0, 1);
            psi0            = g0 ./ g1;
            psi1            = g2 ./ g1;
            psi2            = exp(2 * c * x) .* psi0;
            hratio          = (1 - psi2) ./ (1 + psi2);
            mu_tmp          = signx .* (x - c * hratio);
            sigma_sqr_tmp   = 1 + (1-hratio.^2) * c^2 ...
                - (1+hratio) .* c .* psi1;

            %% --- Estimates and variances
            b2          = s * D2 * mu_tmp;
            varb2       = s_sqr * D2 * diag(sigma_sqr_tmp) * D2';
            if (k1>0)
                Q           = V1r * X1' * X2;
                b1          = V1r * X1' * (y - X2d*b2);
                b1          = D1 * b1;
                varb1       = s_sqr * V1r + s_sqr * Q * diag(sigma_sqr_tmp) * Q';
                varb1       = D1 * varb1 * D1';
            else
                b1          = [];
                varb1       = [];
            end
        end % of ifail(4) == 0
    end % of k2 > 0
end

%% --- Compute output
if (sum(ifail) == 0)
    b  = [b1; b2];
    se = [diag(varb1).^0.5; diag(varb2).^0.5];
else
    b  = nan; 
    se = nan;
end
other  = [];
